'' =================================================================================================
''
''   File....... jm_time.spin
''   Purpose.... Asyncrhonous (accumulator) timer
''               -- methods must be called every ~53.687s (80MHz) or sooner to prevent rollover
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 11 DEC 2014
''
'' =================================================================================================


var

  long  sync                                                    ' sync point (snapshot of cnt)
  long  ustix                                                   ' ticks per microsecond
  long  mstix                                                   ' ticks per millisecond
  long  rawtix                                                  ' timing (ticks) accumulator
  long  msecs                                                   ' milliseconds accumulator
  

pub start 

'' Start or restart the timer object

  return startx(cnt)                                            ' start now
        

pub startx(now)

'' Start the timer object with specific sync point
'' -- for synchronizing with another timer

  sync  := now                                                  ' set sync point
  ustix := clkfreq / 1_000_000                                  ' set ticks/microsecond
  mstix := clkfreq / 1_000                                      ' set ticks/millisecond
  longfill(@rawtix, 0, 2)                                       ' clear rawtix and msecs

  return sync

  
pub millis

'' Returns milliseconds accumulator

  mark                                                          ' update timer

  return msecs                                                  ' return milliseconds


pub seconds

'' Returns seconds (from ms accumulator)

  mark                                                          ' update timer

  return msecs / 1_000                                          ' return seconds


pub adjust(offset)

'' Adjust milliseconds register

  mark

  msecs += offset


pub mark | now, delta

'' Marks the current point
'' -- updates ticks and ms accumulators
'' -- returns current milliseconds

  now := cnt                                                    ' capture cnt
  delta := now - sync                                           ' delta since last capture

  if (delta < 0)                                                ' rollover past posx?
    msecs += posx / mstix                                       ' add posx millis
    delta += posx + (posx // mstix) + 2                         ' correct delta

  rawtix += delta                                               ' increment ticks
  msecs += rawtix / mstix                                       ' update millis
  rawtix //= mstix                                              ' udpate ticks    

  sync := now                                                   ' reset sync point
  
  return msecs                                                  ' return millis


pub get_sync

'' Returns sync point
'' -- used to syncronize multiple timers

  return sync
  

pub pause(ms) | t

'' Pause ms milliseconds
'' -- must be positive!

  t := cnt - 1168                                               ' sync with overhead correction
  repeat ms
    waitcnt(t += mstix)                                         ' wait 1ms


pub pause_us(us)

'' Pause us microseconds
'' -- must be positive, minimum of 50us
'' -- if to low, program will hang for about one minute

  waitcnt(cnt + (us * ustix) - 992)
  
    
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}